package org.slf4j.bridge;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

public class LogManager extends java.util.logging.LogManager {
	protected Map levelToLogMsg = new HashMap();
	protected Map levelMap = new HashMap();
	
	public LogManager() {}

	public java.util.logging.Logger getLogger(String name) {
		return super.getLogger(name);
	}

	public boolean addLogger(Logger logger) {
		if (!(logger instanceof org.slf4j.bridge.Logger)) {
			logger = new org.slf4j.bridge.Logger(logger.getName(), null, this);
		}
		return super.addLogger(logger);
	}

	public void readConfiguration() throws IOException, SecurityException {
		//configurable via properties file in Thread.currentThread().getContextClassLoader() ? would have to be read for every Logger successfully linked to a slf4j logger
		
		//allow for remapping of source jul Level to target jul Level, e.g.:
		//levelMap.put(Level.FINE, Level.SEVERE);
		//levelMap.put(Level.FINER, Level.SEVERE);
		//levelMap.put(Level.FINEST, Level.SEVERE);
		
		//allow for mapping of target jul Level to slf4j methods
		levelToLogMsg.put(Level.ALL, "trace"); // necessary?
		levelToLogMsg.put(Level.FINEST, "trace");
		levelToLogMsg.put(Level.FINER, "debug");
		levelToLogMsg.put(Level.FINE, "debug");
		levelToLogMsg.put(Level.INFO, "info");
		levelToLogMsg.put(Level.WARNING, "warn");
		levelToLogMsg.put(Level.SEVERE, "error");
		levelToLogMsg.put(null, "trace");
		
		//super.readConfiguration();
	}
	
	public void readConfiguration(InputStream ins) throws IOException, SecurityException {
		readConfiguration();
		//super.readConfiguration(ins);
	}

	protected String isEnabledMethodNameForLevel(Level level) {
		String slf4jLevel = logMethodNameForLevel(level);
		return "is" + slf4jLevel.substring(0, 1).toUpperCase() + slf4jLevel.substring(1) + "Enabled";
	}
	
	protected String logMethodNameForLevel(Level level) {
		String slf4jLevel = (String) levelToLogMsg.get(level);
		if (slf4jLevel == null) slf4jLevel = (String) levelToLogMsg.get(null);
		if (slf4jLevel == null) slf4jLevel = "trace";
		return slf4jLevel;
	}

	public Level mappedToLevel(Level level) {
		Level target = (Level) levelMap.get(level);
		return target == null ? level : target;
	}
}
