package org.slf4j.bridge;

import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.ResourceBundle;
import java.util.logging.Filter;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;

public class Logger extends java.util.logging.Logger {
	public static class JulToSlf4jException extends Exception {
		private static final long serialVersionUID = 1L;

		public JulToSlf4jException() {
			super();
		}

		public JulToSlf4jException(String message, Throwable cause) {
			super(message, cause);
		}

		public JulToSlf4jException(String message) {
			super(message);
		}

		public JulToSlf4jException(Throwable cause) {
			super(cause);
		}
	};
	
	public static final Logger global = new Logger(GLOBAL_LOGGER_NAME, null);
	
	protected LogManager logManager;
	
	protected volatile Object logger;
	
	protected Logger(String name, String resourceBundleName) {
		this(name, resourceBundleName, (LogManager) LogManager.getLogManager());
	}

	public Logger(String name, String resourceBundleName, LogManager logManager) {
		super(name, resourceBundleName);
		this.logManager = logManager;
//		try {
//			throw new Exception("name='" + name + "'");
//		} catch(Exception e) {
//			e.printStackTrace();
//		}
	}

	public void addHandler(Handler handler) throws SecurityException {
		super.addHandler(handler);
	}

	public void config(String msg) {
		log(Level.FINEST, msg);
	}

	public void entering(String sourceClass, String sourceMethod, Object param1) {
		log(Level.FINEST, sourceClass + "::" + sourceMethod + "(" + param1 + ")");
	}

	public void entering(String sourceClass, String sourceMethod, Object[] params) {
		log(Level.FINEST, sourceClass + "::" + sourceMethod + "(" + Arrays.toString(params) + ")");
	}

	public void entering(String sourceClass, String sourceMethod) {
		log(Level.FINEST, sourceClass + "::" + sourceMethod + "()");
	}

	public void exiting(String sourceClass, String sourceMethod, Object result) {
		log(Level.FINEST, sourceClass + "::" + sourceMethod + "():" + result);
	}

	public void exiting(String sourceClass, String sourceMethod) {
		log(Level.FINEST, sourceClass + "::" + sourceMethod + "():");
	}

	public void fine(String msg) {
		log(Level.FINE, msg);
	}

	public void finer(String msg) {
		log(Level.FINER, msg);
	}

	public void finest(String msg) {
		log(Level.FINEST, msg);
	}

	public Filter getFilter() {
		return super.getFilter();
	}

	public Handler[] getHandlers() {
		return super.getHandlers();
	}

	public Level getLevel() {
		return super.getLevel();
	}

	public java.util.logging.Logger getParent() {
		return super.getParent();
	}

	public ResourceBundle getResourceBundle() {
		return super.getResourceBundle();
	}

	public String getResourceBundleName() {
		return super.getResourceBundleName();
	}

	public boolean getUseParentHandlers() {
		return super.getUseParentHandlers();
	}

	public void info(String msg) {
		log(Level.INFO, msg);
	}

	public boolean isLoggable(Level level) {
		Object logger = this.logger;
		if (logger == null) logger = captureSlf4jLogger();
		if (logger == null) return super.isLoggable(level);
		try {
			level = logManager.mappedToLevel(level);
			return ((Boolean)invokeSlf4jLogger(this.logManager.isEnabledMethodNameForLevel(level))).booleanValue();
		} catch(JulToSlf4jException e) {
			//e.printStackTrace();
			return super.isLoggable(level);
		}
	}

	public void log(Level level, String msg, Object param1) {
		Object logger = this.logger;
		if (logger == null) logger = captureSlf4jLogger();
		if (logger == null) {
			super.log(level, msg, param1);
		} else
			try {
				level = logManager.mappedToLevel(level);
				Object[] p = new Object[] { msg, param1 };
				StringBuilder sb = new StringBuilder("{} {}");
				invokeSlf4jLogger(this.logManager.logMethodNameForLevel(level), new Class[] { String.class, Object.class }, new Object[] { sb.toString(), p });
			} catch(JulToSlf4jException e) {
				//e.printStackTrace();
				super.log(level, msg, param1);
			}
	}

	public void log(Level level, String msg, Object[] params) {
		Object logger = this.logger;
		if (logger == null) logger = captureSlf4jLogger();
		if (logger == null) {
			super.log(level, msg, params);
		} else
			try {
				level = logManager.mappedToLevel(level);
				StringBuilder sb = new StringBuilder(params.length + params.length + params.length + 2).append("{}");
				Object[] p = new Object[params.length + 1];
				for(int i = 0; i < params.length; i++) {
					p[i + 1] = params[i];
					sb.append(" {}");
				}
				p[0] = msg;
				invokeSlf4jLogger(this.logManager.logMethodNameForLevel(level), new Class[] { String.class, Object.class }, new Object[] { sb.toString(), p });
			} catch(JulToSlf4jException e) {
				//e.printStackTrace();
				super.log(level, msg, params);
			}
	}

	public void log(Level level, String msg, Throwable thrown) {
		Object logger = this.logger;
		if (logger == null) logger = captureSlf4jLogger();
		if (logger == null) {
			super.isLoggable(level);
		} else
			try {
				level = logManager.mappedToLevel(level);
				invokeSlf4jLogger(this.logManager.logMethodNameForLevel(level), new Class[] { String.class, Throwable.class }, new Object[] { msg, thrown });
			} catch(JulToSlf4jException e) {
				//e.printStackTrace();
				super.log(level, msg, thrown);
			}
	}

	public void log(Level level, String msg) {
		Object logger = this.logger;
		if (logger == null) logger = captureSlf4jLogger();
		if (logger == null) {
			super.log(level, msg);
		} else
			try {
				level = logManager.mappedToLevel(level);
				invokeSlf4jLogger(this.logManager.logMethodNameForLevel(level), new Class[] { String.class }, new Object[] { msg });
			} catch(JulToSlf4jException e) {
				//e.printStackTrace();
				super.log(level, msg);
			}
	}

	public void log(LogRecord record) {
		//log(record.getLevel(), record.getMessage(), record.getParameters(), record.getThrown());
		log(record.getLevel(), record.getMessage(), record.getThrown());
	}

	public void logp(Level level, String sourceClass, String sourceMethod, String msg, Object param1) {
		log(level, msg, param1);
		//delegate.logp(level, sourceClass, sourceMethod, msg, param1);
	}

	public void logp(Level level, String sourceClass, String sourceMethod, String msg, Object[] params) {
		log(level, msg, params);
		//delegate.logp(level, sourceClass, sourceMethod, msg, params);
	}

	public void logp(Level level, String sourceClass, String sourceMethod, String msg, Throwable thrown) {
		log(level, msg, thrown);
		//delegate.logp(level, sourceClass, sourceMethod, msg, thrown);
	}

	public void logp(Level level, String sourceClass, String sourceMethod, String msg) {
		log(level, msg);
		//delegate.logp(level, sourceClass, sourceMethod, msg);
	}

	public void logrb(Level level, String sourceClass, String sourceMethod, String bundleName, String msg, Object param1) {
		logp(level, sourceClass, sourceMethod, msg, param1);
		//delegate.logrb(level, sourceClass, sourceMethod, bundleName, msg, param1);
	}

	public void logrb(Level level, String sourceClass, String sourceMethod, String bundleName, String msg, Object[] params) {
		logp(level, sourceClass, sourceMethod, msg, params);
		//delegate.logrb(level, sourceClass, sourceMethod, bundleName, msg, params);
	}

	public void logrb(Level level, String sourceClass, String sourceMethod, String bundleName, String msg, Throwable thrown) {
		logp(level, sourceClass, sourceMethod, msg, thrown);
		//delegate.logrb(level, sourceClass, sourceMethod, bundleName, msg, thrown);
	}

	public void logrb(Level level, String sourceClass, String sourceMethod, String bundleName, String msg) {
		logp(level, sourceClass, sourceMethod, bundleName, msg);
		//delegate.logrb(level, sourceClass, sourceMethod, bundleName, msg);
	}

	public void removeHandler(Handler handler) throws SecurityException {
		super.removeHandler(handler);
	}

	public void severe(String msg) {
		log(Level.SEVERE, msg);
	}

	public void throwing(String sourceClass, String sourceMethod, Throwable thrown) {
		log(Level.FINEST, sourceClass + "::" + sourceMethod + "():", thrown);
	}

	public void warning(String msg) {
		log(Level.WARNING, msg);
	}
	
	protected Object captureSlf4jLogger() {
		try {
			ClassLoader cl = Thread.currentThread().getContextClassLoader();
			Object logger = cl.loadClass("org.slf4j.LoggerFactory").getMethod("getLogger", new Class[] { String.class }).invoke(null, new Object[] { getName() });
			synchronized (this) {
				//potential memory leak?
				if (this.logger == null) this.logger = logger;
			}
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			//e.printStackTrace();
		}
		return this.logger;
	}
	
	protected Object invokeSlf4jLogger(String methodName) throws JulToSlf4jException {
		return invokeSlf4jLogger(methodName, new Class[0], new Object[0]);
	}
	
	protected Object invokeSlf4jLogger(String methodName, Class[] paramTypes, Object[] params) throws JulToSlf4jException {
		try {
			if (logger == null) throw new JulToSlf4jException();
			Object result = logger.getClass().getMethod(methodName, paramTypes).invoke(logger, params);
			//System.err.println(this + "[" + getName() + "]" + methodName + "(" + Arrays.toString(paramTypes) + ") invoked with (" + Arrays.toString(params) + "):" + result);
			return result;
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
			throw new JulToSlf4jException(e);
		} catch (SecurityException e) {
			e.printStackTrace();
			throw new JulToSlf4jException(e);
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			throw new JulToSlf4jException(e);
		} catch (InvocationTargetException e) {
			e.printStackTrace();
			throw new JulToSlf4jException(e);
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
			throw new JulToSlf4jException(e);
		}
	}
}
